Package: ubiquity
Type: Package
Title: PKPD, PBPK, and Systems Pharmacology Modeling Tools
Version: 2.1.0
Authors@R: c(person("John", "Harrold", 
                     role    = c("aut", "cre"),
                     email   = "john.m.harrold@gmail.com",
                     comment = c(ORCID="0000-0003-2052-4373")))
Maintainer: John Harrold <john.m.harrold@gmail.com>
Description: Complete work flow for the analysis of pharmacokinetic pharmacodynamic (PKPD), physiologically-based pharmacokinetic (PBPK) and systems pharmacology models including: creation of ordinary differential equation-based models, pooled parameter estimation, individual/population based simulations, rule-based simulations for clinical trial design and modeling assays, deployment with a customizable 'Shiny' app, and non-compartmental analysis. System-specific analysis templates can be generated and each element includes integrated reporting with 'PowerPoint' and 'Word'. 
URL: https://r.ubiquity.tools
SystemRequirements: Perl
BugReports: https://github.com/john-harrold/ubiquity/issues
License: BSD_2_clause + file LICENSE
Encoding: UTF-8
Depends: R (>= 4.2.0)
Imports: cli, deSolve, dplyr (>= 1.0.0), digest, doParallel, flextable,
        foreach, ggplot2, knitr, MASS, onbrand (>= 1.0.2), optimx,
        PKNCA, pso, readxl, rmarkdown, rhandsontable, scales, stats,
        stringr, shiny,
Suggests: babelmixr2, GA, GGally, gridGraphics, gridExtra, grid,
        officer, rxode2, webshot, ggrepel, rstudioapi, testthat
VignetteBuilder: knitr
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2025-01-06 23:16:53 UTC; jmh
Author: John Harrold [aut, cre] (<https://orcid.org/0000-0003-2052-4373>)
Repository: CRAN
Date/Publication: 2025-01-07 14:30:02 UTC
Built: R 4.6.0; ; 2026-04-30 17:58:34 UTC; unix
