# Generated by roxygen2: do not edit by hand

export(GUI_log_entry)
export(apply_sub_file_COV)
export(build_system)
export(calculate_halflife)
export(calculate_objective)
export(fetch_full_parameters)
export(find_bracketed_arguments)
export(generate_parameter)
export(generate_subject)
export(gg_axis)
export(gg_log10_xaxis)
export(gg_log10_yaxis)
export(linspace)
export(logspace)
export(nm_select_records)
export(pad_string)
export(parse_patterns)
export(prepare_figure)
export(run_simulation_titrate)
export(run_simulation_ubiquity)
export(simulate_subjects)
export(som_to_df)
export(system_check_requirements)
export(system_check_steady_state)
export(system_clear_cohorts)
export(system_define_cohort)
export(system_define_cohorts_nm)
export(system_estimate_parameters)
export(system_fetch_TSsys)
export(system_fetch_guess)
export(system_fetch_iiv)
export(system_fetch_nca)
export(system_fetch_nca_columns)
export(system_fetch_parameters)
export(system_fetch_rpt_officer_object)
export(system_fetch_rpt_onbrand_object)
export(system_fetch_set)
export(system_fetch_template)
export(system_glp_init)
export(system_glp_scenario)
export(system_load_data)
export(system_log_debug_save)
export(system_log_entry)
export(system_log_init)
export(system_nca_parameters_meta)
export(system_nca_run)
export(system_nca_summary)
export(system_new)
export(system_new_list)
export(system_new_tt_rule)
export(system_od_general)
export(system_plot_cohorts)
export(system_rpt_add_doc_content)
export(system_rpt_add_slide)
export(system_rpt_estimation)
export(system_rpt_nca)
export(system_rpt_read_template)
export(system_rpt_save_report)
export(system_rpt_template_details)
export(system_select_set)
export(system_set_bolus)
export(system_set_covariate)
export(system_set_guess)
export(system_set_iiv)
export(system_set_option)
export(system_set_parameter)
export(system_set_rate)
export(system_set_rpt_officer_object)
export(system_set_rpt_onbrand_object)
export(system_set_tt_bolus)
export(system_set_tt_cond)
export(system_set_tt_rate)
export(system_simulate_estimation_results)
export(system_ts_to_simtime)
export(system_view)
export(system_zero_inputs)
export(tic)
export(timecourse_stats)
export(toc)
export(var2string)
export(var2string_gen)
export(vp)
export(workshop_fetch)
import(cli)
import(deSolve)
import(doParallel)
import(foreach)
import(ggplot2)
import(knitr)
import(onbrand)
import(optimx)
import(pso)
import(rhandsontable)
import(rmarkdown)
import(stringr)
importFrom(MASS,mvrnorm)
importFrom(PKNCA,PKNCA.options)
importFrom(PKNCA,PKNCAconc)
importFrom(PKNCA,PKNCAdata)
importFrom(PKNCA,PKNCAdose)
importFrom(PKNCA,get.interval.cols)
importFrom(PKNCA,pk.nca)
importFrom(digest,digest)
importFrom(dplyr,all_of)
importFrom(dplyr,select)
importFrom(flextable,add_footer)
importFrom(flextable,add_header)
importFrom(flextable,align)
importFrom(flextable,as_chunk)
importFrom(flextable,as_paragraph)
importFrom(flextable,autofit)
importFrom(flextable,body_add_flextable)
importFrom(flextable,delete_part)
importFrom(flextable,merge_h)
importFrom(flextable,regulartable)
importFrom(flextable,set_header_labels)
importFrom(flextable,theme_alafoli)
importFrom(flextable,theme_booktabs)
importFrom(flextable,theme_box)
importFrom(flextable,theme_tron)
importFrom(flextable,theme_tron_legacy)
importFrom(flextable,theme_vader)
importFrom(flextable,theme_vanilla)
importFrom(flextable,theme_zebra)
importFrom(parallel,makeCluster)
importFrom(parallel,stopCluster)
importFrom(readxl,read_xls)
importFrom(readxl,read_xlsx)
importFrom(scales,math_format)
importFrom(scales,squish_infinite)
importFrom(scales,trans_format)
importFrom(stats,median)
importFrom(stats,qt)
importFrom(stats,sd)
importFrom(stats,var)
importFrom(utils,capture.output)
importFrom(utils,packageVersion)
importFrom(utils,read.csv)
importFrom(utils,read.delim)
importFrom(utils,sessionInfo)
importFrom(utils,setTxtProgressBar)
importFrom(utils,tail)
importFrom(utils,txtProgressBar)
importFrom(utils,write.csv)
