The packages in this directory (under source/) are verbatim copies of normal
source packages that form part of the installer (debian-installer or d-i).
If you're uploading to the Ubuntu archive, please do not change the copies
here; instead, upload changes to the source packages themselves, wait for
the source uploads to reach the archive, and then run 'debian/rules update'.
