This udeb provides partman with new method "efi" (Use the partition
as an EFI System Partition).  It also provides an imaginary file system
called also "efi" that is automatically selected for all partitions
used with method efi.
