This package is "arch: arm mipsel" but it can be built on any architecture.
Simply call: "dpkg-buildpackage -amipsel -rfakeroot".  The reason the
package is not "arch: any" is because it's only useful on one ARM and one
mipsel sub-arch.
