This package installs the full Unifont on /lib/unifont.bgf

The font is not reduced, as it cannot be known at build-time of this
package what udebs will be used when installing, and because if we
run base-config from within a chroot, we may encounter arbitrary strings
in _their_ debconf templates.

It might be possible to minimise the font by limiting it to coverage
for given languages -- eg remove heiroglyphics, if present.
Need to investigate.
