netcfg

This package is used by debian-installer to configure the network. The purpose
is two-fold:

1. configure the network for the currently running kernel so that the base
system can be retrieved from the internet or a local mirror.
2. write configuration files so that when the machine reboots into the full
debian system the network is brought up automatically.

netcfg is written specifically for the debian-installer and probably won't work
as expected if you try to run it on a full Debian system.  If you want
something for a full system, take a look at etherconf or pppconfig (eventually).


Currently static and dhcp configuration is supported.


