This package adds to the main partitioning menu of partman an item
`Automatically partition the storage devices'.  For each free space
it also adds an item `Automatically partition this free space'.

For a description of the format and use of recipe files and other important
documentation, see debian-installer/doc/devel/partman-auto-recipe.txt

Note that for architecture-specific recipe files, partman-auto will look 
in the following places for recipes:

/lib/partman/recipes-<arch>-<subarch>/*
/lib/partman/recipes-<arch>/*
/lib/partman/recipes/*

Partman-auto will use the recipes from the first of these directories
that exists.
