As of apt 0.7.9ubuntu5, apt will refuse to downgrade from an authenticated
repository to an unauthenticated repository, preventing some possible
attacks from malicious mirrors. In order to ensure that some repositories
are safe right from initial installation, we provide sample signed Release
files for them. The age of these Release files does not matter; all that
matters is that they were downloaded from the archive in question at some
point, and that they are signed by a currently valid key.
